/*this project was developed by the people mentioned in the following link:
www.picprojects.net/rc5_decoder/index.html
Modifications to the project were made to make it functional
with Bolt 18F2550 system and its bootloader software.
To test the program, you will need a Bolt 18F2550 board,
any Philips RC5 infrared remote control, and an interface circuit
with an IR receiver TFM5360 (or equivalent) inserted to the Bolt board. 
Please refer for complete description of project to www.puntoflotante.net*/

#include <p18cxxx.h>
#include <usart.h>
#include <stdio.h>
#include <timers.h>
#include "SERIAL-PORT-BOLT.h"
#include "rc5.h"

/** I N T E R R U P T S  *******************************************/
void InterruptHandler (void); //prototype

extern void _startup( void ); // See c018i.c in your C18 compiler dir 
#pragma code _RESET_INTERRUPT_VECTOR = 0x000800 
void _reset( void ) 
{ 
    _asm goto _startup _endasm 
} 

#pragma code _HIGH_INTERRUPT_VECTOR = 0x000808
void _high_ISR (void)
{
    _asm
        goto InterruptHandler       // Sprung zur Interruptroutine
    _endasm
}

#pragma code _LOW_INTERRUPT_VECTOR = 0x000818
void _low_ISR (void)
{
    _asm
        goto InterruptHandler       // Sprung zur Interruptroutine
    _endasm
}

#pragma code
#define LED_On()		TRISBbits.TRISB0 = 0; LATBbits.LATB0 = 1
#define LED_Off()		TRISBbits.TRISB0 = 0; LATBbits.LATB0 = 0

#pragma interruptlow InterruptHandler
void InterruptHandler (void)
{
    if( INTCONbits.RBIF )    // PORTB Change Interrupt
    {
		LED_On();
		RC5_InterruptHandler();
		LED_Off();
		INTCONbits.RBIF = 0;  // Wenn ja Interruptflag lschen
    }
}

#pragma code

void main(void)
{
	uint16 rc5code;
	uint8 dummy;

	TRISC = 0xBF;
 	TRISBbits.RB4=1;
    
	// configure timer 0
	OpenTimer0(TIMER_INT_OFF & T0_8BIT & T0_SOURCE_INT & T0_PS_1_256);

    	activa_serial();

	INTCONbits.RBIE = 1;     // Enable Port-b change interrupt
	dummy = RC5_GetPin();		// clear mismatch
	INTCONbits.RBIF = 0;  // Wenn ja Interruptflag lschen
	INTCONbits.GIE = 1;

	LED_On();
	printf("*** C18 RC5 Demo! ***\n\r");
	LED_Off();

	while(1)
	{
        /* check if new data is available */
        if (RC5_CodeReady())
        {
            /* get code */
            rc5code = RC5_GetCode();
            
            printf("RC5 Code=0x%04X DevAddr=0x%02X Toggle=%d Cmd=0x%02X\n\r",
                   rc5code,
                   RC5_GetDeviceAddr(rc5code),
                   RC5_GetToggleBit(rc5code),
                   RC5_GetCmd(rc5code));
        }
        
        /* this function increments the RC5 timeout timer */
        /* NOTE: Decoding will also work without calling this function, but */
        /*       it could happen that RC5 codes are sometimes not getting */
        /*       recognized because of decoding state machine stucks due */
        /*       to erroneous RC5 signal. */
        RC5_TimeoutIncrement();

	}
}